@@ -38,9 +38,11 @@ our $VERSION = $AnyEvent::VERSION;
 
 # some public dns servers
 our @DNS_FALLBACK = (
-   v209.244.0.3, v209.244.0.4, # level3
-   v4.2.2.1, v4.2.2.3, v4.2.2.4, v4.2.2.5, v4.2.2.6, # vnsc-pri.sys.gtei.net
+   (v8.8.8.8, v8.8.4.4)[rand 2], # google public dns
+   (v209.244.0.3, v209.244.0.4)[rand 2], # level3
+   (v4.2.2.1, v4.2.2.3, v4.2.2.4, v4.2.2.5, v4.2.2.6)[rand 4], # vnsc-pri.sys.gtei.net
 );
+push @DNS_FALLBACK, splice @DNS_FALLBACK, rand $_, 1 for reverse 1..@DNS_FALLBACK;
 
 =item AnyEvent::DNS::a $domain, $cb->(@addrs)
 
@@ -61,7 +63,16 @@ Tries to resolve the given domain name into a list of name servers.
 
 =item AnyEvent::DNS::txt $domain, $cb->(@hostnames)
 
-Tries to resolve the given domain name into a list of text records.
+Tries to resolve the given domain name into a list of text records. Only
+the first text string per record will be returned. If you want all
+strings, you need to call the resolver manually:
+
+   resolver->resolve ($domain => "txt", sub {
+      for my $record (@_) {
+         my (undef, undef, undef, @txt) = @$record;
+         # strings now in @txt
+      }
+   });
 
 =item AnyEvent::DNS::srv $service, $proto, $domain, $cb->(@srv_rr)
 
@@ -79,21 +90,21 @@ Example:
    AnyEvent::DNS::srv "sip", "udp", "schmorp.de", sub { ...
    # @_ = ( [10, 10, 5060, "sip1.schmorp.de" ] )
 
+=item AnyEvent::DNS::any $domain, $cb->(@rrs)
+
+Tries to resolve the given domain and passes all resource records found to
+the callback.
+
 =item AnyEvent::DNS::ptr $domain, $cb->(@hostnames)
 
 Tries to make a PTR lookup on the given domain. See C<reverse_lookup>
 and C<reverse_verify> if you want to resolve an IP address to a hostname
 instead.
 
-=item AnyEvent::DNS::any $domain, $cb->(@rrs)
-
-Tries to resolve the given domain and passes all resource records found to
-the callback.
-
 =item AnyEvent::DNS::reverse_lookup $ipv4_or_6, $cb->(@hostnames)
 
 Tries to reverse-resolve the given IPv4 or IPv6 address (in textual form)
-into it's hostname(s). Handles V4MAPPED and V4COMPAT IPv6 addresses
+into its hostname(s). Handles V4MAPPED and V4COMPAT IPv6 addresses
 transparently.
 
 =item AnyEvent::DNS::reverse_verify $ipv4_or_6, $cb->(@hostnames)
@@ -109,7 +120,7 @@ address you originally resolved.
 
 Example:
 
-   AnyEvent::DNS::ptr "2001:500:2f::f", sub { print shift };
+   AnyEvent::DNS::reverse_verify "2001:500:2f::f", sub { print shift };
    # => f.root-servers.net
 
 =cut
@@ -124,7 +135,7 @@ sub a($$) {
    my ($domain, $cb) = @_;
 
    resolver->resolve ($domain => "a", sub {
-      $cb->(map $_->[3], @_);
+      $cb->(map $_->[4], @_);
    });
 }
 
@@ -132,7 +143,7 @@ sub aaaa($$) {
    my ($domain, $cb) = @_;
 
    resolver->resolve ($domain => "aaaa", sub {
-      $cb->(map $_->[3], @_);
+      $cb->(map $_->[4], @_);
    });
 }
 
@@ -140,7 +151,7 @@ sub mx($$) {
    my ($domain, $cb) = @_;
 
    resolver->resolve ($domain => "mx", sub {
-      $cb->(map $_->[4], sort { $a->[3] <=> $b->[3] } @_);
+      $cb->(map $_->[5], sort { $a->[4] <=> $b->[4] } @_);
    });
 }
 
@@ -148,7 +159,7 @@ sub ns($$) {
    my ($domain, $cb) = @_;
 
    resolver->resolve ($domain => "ns", sub {
-      $cb->(map $_->[3], @_);
+      $cb->(map $_->[4], @_);
    });
 }
 
@@ -156,7 +167,7 @@ sub txt($$) {
    my ($domain, $cb) = @_;
 
    resolver->resolve ($domain => "txt", sub {
-      $cb->(map $_->[3], @_);
+      $cb->(map $_->[4], @_);
    });
 }
 
@@ -169,7 +180,7 @@ sub srv($$$$) {
 
       # classify by priority
       my %pri;
-      push @{ $pri{$_->[3]} }, [ @$_[3,4,5,6] ]
+      push @{ $pri{$_->[4]} }, [ @$_[4,5,6,7] ]
          for @_;
 
       # order by priority
@@ -199,7 +210,7 @@ sub ptr($$) {
    my ($domain, $cb) = @_;
 
    resolver->resolve ($domain => "ptr", sub {
-      $cb->(map $_->[3], @_);
+      $cb->(map $_->[4], @_);
    });
 }
 
@@ -243,7 +254,7 @@ sub reverse_lookup($$) {
       or return $cb->();
 
    resolver->resolve ($ip => "ptr", sub {
-      $cb->(map $_->[3], @_);
+      $cb->(map $_->[4], @_);
    });
 }
 
@@ -271,7 +282,7 @@ sub reverse_verify($$) {
          resolver->resolve ("$name." => ($af == (AF_INET) ? "a" : "aaaa"), sub {
             for (@_) {
                push @res, $name
-                  if $_->[3] eq $ip;
+                  if $_->[4] eq $ip;
             }
             $cb->(@res) unless --$cnt;
          });
@@ -392,7 +403,7 @@ if ($] < 5.008) {
 sub _enc_qd() {
    (_enc_name $_->[0]) . pack "nn",
      ($_->[1] > 0 ? $_->[1] : $type_id {$_->[1]}),
-     ($_->[2] > 0 ? $_->[2] : $class_id{$_->[2] || "in"})
+     ($_->[3] > 0 ? $_->[2] : $class_id{$_->[2] || "in"})
 }
 
 sub _enc_rr() {
@@ -536,6 +547,7 @@ sub _dec_rr {
       $name,
       $type_str{$rt}  || $rt,
       $class_str{$rc} || $rc,
+      $ttl,
       ($dec_rr{$rt} || sub { $_ })->(),
    ]
 }
@@ -558,6 +570,7 @@ Examples:
                  'uni-karlsruhe.de',
                  'soa',
                  'in',
+                 600,
                  'netserv.rz.uni-karlsruhe.de',
                  'hostmaster.rz.uni-karlsruhe.de',
                  2008052201, 10800, 1800, 2592000, 86400
@@ -579,13 +592,13 @@ Examples:
      'qd' => [ [ 'www.google.de', 'a', 'in' ] ],
      'rc' => 0,
      'ar' => [
-               [ 'a.l.google.com', 'a', 'in', '209.85.139.9' ],
-               [ 'b.l.google.com', 'a', 'in', '64.233.179.9' ],
-               [ 'c.l.google.com', 'a', 'in', '64.233.161.9' ],
+               [ 'a.l.google.com', 'a', 'in', 3600, '209.85.139.9' ],
+               [ 'b.l.google.com', 'a', 'in', 3600, '64.233.179.9' ],
+               [ 'c.l.google.com', 'a', 'in', 3600, '64.233.161.9' ],
              ],
      'ns' => [
-               [ 'l.google.com', 'ns', 'in', 'a.l.google.com' ],
-               [ 'l.google.com', 'ns', 'in', 'b.l.google.com' ],
+               [ 'l.google.com', 'ns', 'in', 3600, 'a.l.google.com' ],
+               [ 'l.google.com', 'ns', 'in', 3600, 'b.l.google.com' ],
              ],
      'tc' => '',
      'ra' => 1,
@@ -593,10 +606,10 @@ Examples:
      'id' => 64265,
      'aa' => '',
      'an' => [
-               [ 'www.google.de', 'cname', 'in', 'www.google.com' ],
-               [ 'www.google.com', 'cname', 'in', 'www.l.google.com' ],
-               [ 'www.l.google.com', 'a', 'in', '66.249.93.104' ],
-               [ 'www.l.google.com', 'a', 'in', '66.249.93.147' ],
+               [ 'www.google.de', 'cname', 'in', 3600, 'www.google.com' ],
+               [ 'www.google.com', 'cname', 'in', 3600, 'www.l.google.com' ],
+               [ 'www.l.google.com', 'a', 'in', 3600, '66.249.93.104' ],
+               [ 'www.l.google.com', 'a', 'in', 3600, '66.249.93.147' ],
              ],
      'rd' => 1,
      'op' => 0
@@ -651,10 +664,11 @@ our $NOW;
 =item AnyEvent::DNS::resolver
 
 This function creates and returns a resolver that is ready to use and
-should mimic the default resolver for your system as good as possible.
+should mimic the default resolver for your system as good as possible. It
+is used by AnyEvent itself as well.
 
-It only ever creates one resolver and returns this one on subsequent
-calls.
+It only ever creates one resolver and returns this one on subsequent calls
+- see C<$AnyEvent::DNS::RESOLVER>, below, for details.
 
 Unless you have special needs, prefer this function over creating your own
 resolver object.
@@ -668,6 +682,16 @@ C<os_config> will be used for OS-specific configuration, unless
 C<$ENV{PERL_ANYEVENT_RESOLV_CONF}> is specified, in which case that file
 gets parsed.
 
+=item $AnyEvent::DNS::RESOLVER
+
+This variable stores the default resolver returned by
+C<AnyEvent::DNS::resolver>, or C<undef> when the default resolver hasn't
+been instantiated yet.
+
+One can provide a custom resolver (e.g. one with caching functionality)
+by storing it in this variable, causing all subsequent resolves done via
+C<AnyEvent::DNS::resolver> to be done via the custom one.
+
 =cut
 
 our $RESOLVER;
@@ -800,7 +824,7 @@ sub new {
 Parses the given string as if it were a F<resolv.conf> file. The following
 directives are supported (but not necessarily implemented).
 
-C<#>-style comments, C<nameserver>, C<domain>, C<search>, C<sortlist>,
+C<#>- and C<;>-style comments, C<nameserver>, C<domain>, C<search>, C<sortlist>,
 C<options> (C<timeout>, C<attempts>, C<ndots>).
 
 Everything else is silently ignored.
@@ -816,7 +840,7 @@ sub parse_resolv_conf {
    my $attempts;
 
    for (split /\n/, $resolvconf) {
-      s/#.*$//; # not quite legal, but many people insist
+      s/\s*[;#].*$//; # not quite legal, but many people insist
 
       if (/^\s*nameserver\s+(\S+)\s*$/i) {
          my $ip = $1;
@@ -825,7 +849,7 @@ sub parse_resolv_conf {
          } else {
             warn "nameserver $ip invalid and ignored\n";
          }
-      } elsif (/^\s*domain\s+(\S*)\s+$/i) {
+      } elsif (/^\s*domain\s+(\S*)\s*$/i) {
          $self->{search} = [$1];
       } elsif (/^\s*search\s+(.*?)\s*$/i) {
          $self->{search} = [split /\s+/, $1];
@@ -843,6 +867,8 @@ sub parse_resolv_conf {
                # debug, rotate, no-check-names, inet6
             }
          }
+      } else {
+         # silently skip stuff we don't understand
       }
    }
 
@@ -877,7 +903,7 @@ sub os_config {
    $self->{search} = [];
 
    if ((AnyEvent::WIN32 || $^O =~ /cygwin/i)) {
-      no strict 'refs';
+      #no strict 'refs';
 
       # there are many options to find the current nameservers etc. on windows
       # all of them don't work consistently:
@@ -938,8 +964,8 @@ sub os_config {
          }
       }
 
-      # always add one fallback server
-      push @{ $self->{server} }, $DNS_FALLBACK[rand @DNS_FALLBACK];
+      # always add the fallback servers
+      push @{ $self->{server} }, @DNS_FALLBACK;
 
       $self->_compile;
    } else {
@@ -1091,7 +1117,7 @@ sub _exec {
                      &$do_retry;
                   };
 
-               $handle->push_write (pack "n/a", $req->[0]);
+               $handle->push_write (pack "n/a*", $req->[0]);
                $handle->push_read (chunk => 2, sub {
                   $handle->unshift_read (chunk => (unpack "n", $_[1]), sub {
                      undef $handle;
@@ -1123,7 +1149,7 @@ sub _exec {
 sub _scheduler {
    my ($self) = @_;
 
-   no strict 'refs';
+   #no strict 'refs';
 
    $NOW = time;
 
@@ -1173,7 +1199,9 @@ This is the main low-level workhorse for sending DNS requests.
 This function sends a single request (a hash-ref formated as specified
 for C<dns_pack>) to the configured nameservers in turn until it gets a
 response. It handles timeouts, retries and automatically falls back to
-virtual circuit mode (TCP) when it receives a truncated reply.
+virtual circuit mode (TCP) when it receives a truncated reply. It does not
+handle anything else, such as the domain searchlist or relative names -
+use C<< ->resolve >> for that.
 
 Calls the callback with the decoded response packet if a reply was
 received, or no arguments in case none of the servers answered.
@@ -1183,6 +1211,13 @@ received, or no arguments in case none of the servers answered.
 sub request($$) {
    my ($self, $req, $cb) = @_;
 
+   # _enc_name barfs on names that are too long, which is often outside
+   # program control, so check for too long names here.
+   for (@{ $req->{qd} }) {
+      return AE::postpone sub { $cb->(undef) }
+         if 255 < length $_->[0];
+   }
+
    push @{ $self->{queue} }, [dns_pack $req, $cb];
    $self->_scheduler;
 }
@@ -1202,16 +1237,17 @@ none on any error or if the name could not be found.
 
 CNAME chains (although illegal) are followed up to a length of 10.
 
-The callback will be invoked with arraryefs of the form C<[$name, $type,
-$class, @data>], where C<$name> is the domain name, C<$type> a type string
-or number, C<$class> a class name and @data is resource-record-dependent
-data. For C<a> records, this will be the textual IPv4 addresses, for C<ns>
-or C<cname> records this will be a domain name, for C<txt> records these
-are all the strings and so on.
+The callback will be invoked with arraryefs of the form C<[$name,
+$type, $class, $ttl, @data>], where C<$name> is the domain name,
+C<$type> a type string or number, C<$class> a class name, C<$ttl> is the
+remaining time-to-live and C<@data> is resource-record-dependent data, in
+seconds. For C<a> records, this will be the textual IPv4 addresses, for
+C<ns> or C<cname> records this will be a domain name, for C<txt> records
+these are all the strings and so on.
 
 All types mentioned in RFC 1035, C<aaaa>, C<srv>, C<naptr> and C<spf> are
-decoded. All resource records not known to this module will have
-the raw C<rdata> field as fourth entry.
+decoded. All resource records not known to this module will have the raw
+C<rdata> field as fifth array element.
 
 Note that this resolver is just a stub resolver: it requires a name server
 supporting recursive queries, will not do any recursive queries itself and
@@ -1254,15 +1290,15 @@ Examples:
 
    # shortened result:
    # [
-   #   [ 'google.com', 'soa', 'in', 'ns1.google.com', 'dns-admin.google.com',
+   #   [ 'google.com', 'soa', 'in', 3600, 'ns1.google.com', 'dns-admin.google.com',
    #     2008052701, 7200, 1800, 1209600, 300 ],
    #   [
-   #     'google.com', 'txt', 'in',
+   #     'google.com', 'txt', 'in', 3600,
    #     'v=spf1 include:_netblocks.google.com ~all'
    #   ],
-   #   [ 'google.com', 'a', 'in', '64.233.187.99' ],
-   #   [ 'google.com', 'mx', 'in', 10, 'smtp2.google.com' ],
-   #   [ 'google.com', 'ns', 'in', 'ns2.google.com' ],
+   #   [ 'google.com', 'a', 'in', 3600, '64.233.187.99' ],
+   #   [ 'google.com', 'mx', 'in', 3600, 10, 'smtp2.google.com' ],
+   #   [ 'google.com', 'ns', 'in', 3600, 'ns2.google.com' ],
    # ]
 
    # resolve a records:
@@ -1270,7 +1306,7 @@ Examples:
 
    # result:
    # [
-   #   [ 'ruth.schmorp.de', 'a', 'in', '129.13.162.95' ]
+   #   [ 'ruth.schmorp.de', 'a', 'in', 86400, '129.13.162.95' ]
    # ]
 
    # resolve any records, but return only a and aaaa records:
@@ -1283,8 +1319,8 @@ Examples:
 
    # result:
    # [
-   #   [ 'test1.laendle', 'a', 'in', '10.0.0.255' ],
-   #   [ 'test1.laendle', 'aaaa', 'in', '3ffe:1900:4545:0002:0240:0000:0000:f7e1' ]
+   #   [ 'test1.laendle', 'a', 'in', 86400, '10.0.0.255' ],
+   #   [ 'test1.laendle', 'aaaa', 'in', 60, '3ffe:1900:4545:0002:0240:0000:0000:f7e1' ]
    # ]
 
 =cut
@@ -1343,7 +1379,7 @@ sub resolve($%) {
                      or return $do_search->(); # cname chain too long
 
                   $cname = 1;
-                  $name = lc $rr[0][3];
+                  $name = lc $rr[0][4];
 
                } elsif ($cname) {
                   # follow the cname
