@@ -392,12 +392,16 @@ sub DESTROY {
    POE::Kernel->call (${${$_[0]}}, "stop");
 }
 
-sub one_event {
+#sub loop {
+#   POE::Kernel->run;
+#}
+
+sub _poll {
    POE::Kernel->loop_do_timeslice;
 }
 
-sub loop {
-   POE::Kernel->run;
+sub AnyEvent::CondVar::Base::_wait {
+   POE::Kernel->loop_do_timeslice until $_[0]{_ae_sent};
 }
 
 1;
