@@ -133,12 +133,16 @@ sub AnyEvent::Impl::Tk::after::DESTROY {
    Tk::after $mw, cancel => $${$_[0]};
 }
 
-sub one_event {
+#sub loop {
+#   Tk::MainLoop;
+#}
+
+sub _poll {
    Tk::DoOneEvent (0);
 }
 
-sub loop {
-   Tk::MainLoop;
+sub AnyEvent::CondVar::Base::_wait {
+   Tk::DoOneEvent (0) until $_[0]{_ae_sent};
 }
 
 1;
