@@ -251,7 +251,9 @@ sub _fork_schedule {
          my $ww; $ww = AE::io $r, 0, sub {
             my $len = sysread $r, $buf, 65536, length $buf;
 
-            if ($len <= 0) {
+            return unless defined $len or $! != Errno::EINTR;
+
+            if (!$len) {
                undef $ww;
                close $r;
                --$forks;
@@ -293,7 +295,7 @@ sub _fork_schedule {
 
             $len = syswrite $w, $result, $len < 65536 ? $len : 65536, $ofs;
 
-            last if $len <= 0;
+            last unless $len || (!defined $len && $! == Errno::EINTR);
 
             $ofs += $len;
          }
@@ -383,30 +385,32 @@ guard.
 
 =cut
 
-if (!$ENV{PERL_ANYEVENT_AVOID_GUARD} && eval { require Guard; $Guard::VERSION >= 0.5 }) {
-   warn "AnyEvent::Util: using Guard module to implement guards.\n" if $AnyEvent::VERBOSE >= 8;
-   *guard = \&Guard::guard;
-} else {
-   warn "AnyEvent::Util: using pure-perl guard implementation.\n" if $AnyEvent::VERBOSE >= 8;
+BEGIN {
+   if (!$ENV{PERL_ANYEVENT_AVOID_GUARD} && eval { require Guard; $Guard::VERSION >= 0.5 }) {
+      warn "AnyEvent::Util: using Guard module to implement guards.\n" if $AnyEvent::VERBOSE >= 8;
+      *guard = \&Guard::guard;
+   } else {
+      warn "AnyEvent::Util: using pure-perl guard implementation.\n" if $AnyEvent::VERBOSE >= 8;
 
-   *AnyEvent::Util::guard::DESTROY = sub {
-      local $@;
+      *AnyEvent::Util::guard::DESTROY = sub {
+         local $@;
 
-      eval {
-         local $SIG{__DIE__};
-         ${$_[0]}->();
+         eval {
+            local $SIG{__DIE__};
+            ${$_[0]}->();
+         };
+
+         warn "runtime error in AnyEvent::guard callback: $@" if $@;
       };
 
-      warn "runtime error in AnyEvent::guard callback: $@" if $@;
-   };
+      *AnyEvent::Util::guard::cancel = sub ($) {
+         ${$_[0]} = sub { };
+      };
 
-   *AnyEvent::Util::guard::cancel = sub ($) {
-      ${$_[0]} = sub { };
-   };
-
-   *guard = sub (&) {
-      bless \(my $cb = shift), "AnyEvent::Util::guard"
-   };
+      *guard = sub (&) {
+         bless \(my $cb = shift), "AnyEvent::Util::guard"
+      };
+   }
 }
 
 =item AnyEvent::Util::close_all_fds_except @fds
@@ -558,7 +562,7 @@ in the callback and print key and certificate.
                      -newkey rsa:2048 -keyout /dev/fd/3
                      -batch -subj /CN=AnyEvent
                     )],
-      "<", "dev/null",
+      "<", "/dev/null",
       ">" , \my $cert,
       "3>", \my $key,
       "2>", "/dev/null";
@@ -600,14 +604,16 @@ sub run_cmd {
             my $w; $w = AE::io $pr, 0,
                "SCALAR" eq ref $ob
                   ? sub {
-                       sysread $pr, $$ob, 16384, length $$ob
-                          and return;
+                       defined (sysread $pr, $$ob, 16384, length $$ob
+                                and return)
+                          or ($! == Errno::EINTR and return);
                        undef $w; $cv->end;
                     }
                   : sub {
                        my $buf;
-                       sysread $pr, $buf, 16384
-                          and return $ob->($buf);
+                       defined (sysread $pr, $buf, 16384
+                                and return $ob->($buf))
+                          or ($! == Errno::EINTR and return);
                        undef $w; $cv->end;
                        $ob->();
                     }
@@ -642,7 +648,9 @@ sub run_cmd {
             my $w; $w = AE::io $pw, 1, sub {
                my $len = syswrite $pw, $data;
 
-               if ($len <= 0) {
+               return unless defined $len or $! != Errno::EINTR;
+
+               if (!$len) {
                   undef $w; $cv->end;
                } else {
                   substr $data, 0, $len, "";
@@ -763,7 +771,7 @@ sub punycode_decode($) {
 
 Implements the IDNA nameprep normalisation algorithm. Or actually the
 UTS#46 algorithm. Or maybe something similar - reality is complicated
-btween IDNA2003, UTS#46 and IDNA2008. If C<$display> is true then the name
+between IDNA2003, UTS#46 and IDNA2008. If C<$display> is true then the name
 is prepared for display, otherwise it is prepared for lookup (default).
 
 If you have no clue what this means, look at C<idn_to_ascii> instead.
@@ -807,9 +815,9 @@ sub idn_nameprep($;$) {
             # not in valid class, search for mapping
             utf8::encode $chr; # the imap table is in utf-8
             (my $rep = index $uts46_imap, "\x00$chr") >= 0
-               or Carp::croak "$_[0]: disallowed characters during idn_nameprep";
+               or Carp::croak "$_[0]: disallowed characters ($chr) during idn_nameprep" . unpack "H*", $chr;
 
-            (substr $uts46_imap, $rep, 128) =~ /\x00 .[\x80-\xbf]* ([^\x00]+) \x00/x
+            (substr $uts46_imap, $rep, 128) =~ /\x00 .[\x80-\xbf]* ([^\x00]*) \x00/x
                or die "FATAL: idn_nameprep imap table has unexpected contents";
 
             $rep = $1;
@@ -845,7 +853,7 @@ sub idn_nameprep($;$) {
    }gex;
 
    # uts46 verification
-   /\.-|-\.|\.\./
+   /\.-|-\./
       and Carp::croak "$_[0]: invalid hyphens detected during idn_nameprep";
 
    # missing: label begin with combining mark, idna2008 bidi
@@ -892,7 +900,7 @@ sub idn_to_ascii($) {
 
    eval {
       # punycode by label
-      for (split /\./, idn_nameprep $_[0]) {
+      for (split /\./, (idn_nameprep $_[0]), -1) {
          if (/[^\x00-\x7f]/) {
             eval {
                push @output, "xn--" . punycode_encode $_;
@@ -908,6 +916,9 @@ sub idn_to_ascii($) {
       1
    } or return $_[0];
 
+   shift @output
+      while !length $output[0] && @output > 1;
+
    join ".", @output
 }
 
